#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
# numpy_ipps documentation build configuration file.
#
# This file is execfile()d with the current directory set to its
# containing dir.
#
# Note that not all possible configuration values are present in this
# autogenerated file.
#
# All configuration values have a default; values that are commented out
# serve to show the default.

# If extensions (or modules to document with autodoc) are in another directory,
# add these directories to sys.path here. If the directory is relative to the
# documentation root, use os.path.abspath to make it absolute, like shown here.
#

import os
import sys

sys.path.insert(0, os.path.abspath("../.."))

import inspect

import numpy
import numpy_ipps


# -- General configuration ------------------------------------------------

# If your documentation needs a minimal Sphinx version, state it here.
#
# needs_sphinx = '1.0'

# Add any Sphinx extension module names here, as strings. They can be
# extensions coming with Sphinx (named 'sphinx.ext.*') or your custom
# ones.
extensions = [
    "sphinx.ext.autodoc",
    "sphinx.ext.doctest",
    "sphinx.ext.intersphinx",
    "sphinx.ext.todo",
    "sphinx.ext.mathjax",
    "sphinx.ext.ifconfig",
    "sphinx.ext.viewcode",
    "sphinx.ext.githubpages",
    "sphinxcontrib.apidoc",
    "breathe",
    "m2r",
    "rst2pdf.pdfbuilder",
]

# Add any paths that contain templates here, relative to this directory.
templates_path = ["_templates"]

# The suffix(es) of source filenames.
# You can specify multiple suffix as a list of string:
#
# source_suffix = ['.rst', '.md']
source_suffix = [".rst", ".md"]

# The master toctree document.
master_doc = "index"

# General information about the project.
project = numpy_ipps.__title__
copyright = numpy_ipps.__copyright__
author = numpy_ipps.__author__

# The version info for the project you're documenting, acts as replacement for
# |version| and |release|, also used in various other places throughout the
# built documents.
#
# The short X.Y version.
version = ".".join(numpy_ipps.__version__.split(".", 2)[:2])
# The full version, including alpha/beta/rc tags.
release = numpy_ipps.__version__

# The language for content autogenerated by Sphinx. Refer to documentation
# for a list of supported languages.
#
# This is also used if you do content translation via gettext catalogs.
# Usually you set "language" from the command line for these cases.
language = None

# List of patterns, relative to source directory, that match files and
# directories to ignore when looking for source files.
# This patterns also effect to html_static_path and html_extra_path
exclude_patterns = ["_build", "Thumbs.db", ".DS_Store"]

# The name of the Pygments (syntax highlighting) style to use.
pygments_style = "sphinx"

# If true, `todo` and `todoList` produce output, else they produce nothing.
todo_include_todos = False


# -- Options for HTML output ----------------------------------------------

# The theme to use for HTML and HTML Help pages.  See the documentation for
# a list of builtin themes.
#
html_theme = "sphinx_rtd_theme"


# Theme options are theme-specific and customize the look and feel of a theme
# further.  For a list of options available for each theme, see the
# documentation.
#
# html_theme_options = {}

# Add any paths that contain custom static files (such as style sheets) here,
# relative to this directory. They are copied after the builtin static files,
# so a file named "default.css" will overwrite the builtin "default.css".
# html_static_path = ['_static']

# Custom sidebar templates, must be a dictionary that maps document names
# to template names.
#
# The default sidebars (for documents that don't match any pattern) are
# defined by theme itself.  Builtin themes are using these templates by
# default: ``['localtoc.html', 'relations.html', 'sourcelink.html',
# 'searchbox.html']``.
#
# html_sidebars = {}


# -- Options for HTMLHelp output ------------------------------------------

# Output file base name for HTML help builder.
htmlhelp_basename = numpy_ipps.__title__


# -- Options for LaTeX output ---------------------------------------------

latex_elements = {
    # The paper size ('letterpaper' or 'a4paper').
    #
    # 'papersize': 'letterpaper',
    # The font size ('10pt', '11pt' or '12pt').
    #
    # 'pointsize': '10pt',
    # Additional stuff for the LaTeX preamble.
    #
    # 'preamble': '',
    # Latex figure (float) alignment
    #
    # 'figure_align': 'htbp',
}

# Grouping the document tree into LaTeX files. List of tuples
# (source start file, target name, title,
#  author, documentclass [howto, manual, or own class]).
latex_documents = [
    (
        master_doc,
        "{}.tex".format(numpy_ipps.__title__),
        "numpy_ipps Documentation",
        author,
        "manual",
    ),
]


# -- Options for manual page output ---------------------------------------

# One entry per manual page. List of tuples
# (source start file, name, description, authors, manual section).
man_pages = [
    (
        master_doc,
        numpy_ipps.__title__,
        "numpy_ipps Documentation",
        [author],
        1,
    ),
]


# -- Options for Texinfo output -------------------------------------------

# Grouping the document tree into Texinfo files. List of tuples
# (source start file, target name, title, author,
#  dir menu entry, description, category)
texinfo_documents = [
    (
        master_doc,
        numpy_ipps.__title__,
        "numpy_ipps Documentation",
        author,
        numpy_ipps.__title__,
        numpy_ipps.__description__,
        "Miscellaneous",
    ),
]


# -- Options for Epub output -------------------------------------------------

# Bibliographic Dublin Core info.
epub_title = project

# The unique identifier of the text. This can be a ISBN number
# or the project homepage.
#
# epub_identifier = ''

# A unique identification for the text.
#
# epub_uid = ''

# A list of files that should not be packed into the epub file.
epub_exclude_files = ["search.html"]

# -- Extension configuration -------------------------------------------------

# -- Options for intersphinx extension ---------------------------------------

# Example configuration for intersphinx: refer to the Python standard library.
intersphinx_mapping = {"https://docs.python.org/": None}

# -- Options for todo extension ----------------------------------------------

# If true, `todo` and `todoList` produce output, else they produce nothing.
todo_include_todos = True

# -- Configuration for autodoc ----------------------------------------------
autodoc_member_order = "alphabetical"
autodoc_default_options = {
    "members": None,
    "undoc-members": False,
    "show-inheritance": False,
}

apidoc_module_dir = "../../numpy_ipps"
apidoc_output_dir = "api"
apidoc_separate_modules = True
apidoc_module_first = True
apidoc_toc_file = False


def add_dtype_candidates(app, what, name, obj, options, lines):
    options["show-inheritance"] = False

    if not hasattr(obj, "dtype_candidates"):
        return

    for i, line in enumerate(lines):
        if i < 2:
            continue
        lines[i] = "| {}".format(line)

    try:
        lines.append(
            "| Numpy dtype candidates: {}".format(
                ", ".join(
                    numpy.dtype(candidate).name
                    for candidate in obj.dtype_candidates
                )
            )
        )
    except ValueError:
        pass

    if hasattr(obj, "ipps_accuracies"):
        lines.append(
            "| Intel IPP Signal accuracy candidates: {} (default)".format(
                ", ".join(candidate.name for candidate in obj.ipps_accuracies)
            )
        )

    lines.append(
        "| Constructor signature: ``({})``".format(
            ", ".join(
                str(parameter)
                for parameter in inspect.signature(obj).parameters.values()
            )
        )
    )
    lines.append(
        "| Call signature: ``({})``".format(
            ", ".join(
                list(inspect.signature(obj.__call__).parameters.keys())[1:]
            )
        )
    )


def remove_ipps_signature(
    app, what, name, obj, options, signature, return_annotation
):
    if not hasattr(obj, "dtype_candidates"):
        return

    return None, None


def setup(app):
    app.connect("autodoc-process-docstring", add_dtype_candidates)
    app.connect("autodoc-process-signature", remove_ipps_signature)
